from django.contrib import admin

from .models import Campaign


class CampaignAdmin(admin.ModelAdmin):
    list_filter = ('status', )
    list_display = (
        'campaign_id',
        'name',
        'status',
        'total_users',
    )


admin.site.register(Campaign, CampaignAdmin)



from django.conf import settings

from rest_framework.generics import ListAPIView
from rest_framework.permissions import IsAuthenticated

from ...models import Campaign
from ..serializers import CampaignSerializer


class CampaignViewSet(ListAPIView):

    model = Campaign
    queryset = Campaign.objects.filter(status=settings.REFERRAL_CAMPAIGN_STATUS_ACTIVE)
    permission_classes = (IsAuthenticated, )
    serializer_class = CampaignSerializer

    def filter_queryset(self, queryset):
        return queryset.filter(users__email=self.request.user.email)



from rest_framework import serializers

from ..models import Campaign


class CampaignSerializer(serializers.ModelSerializer):
    campaignId = serializers.CharField(source='campaign_id')

    class Meta:
        model = Campaign
        fields = ['name', 'campaignId']



from django.apps import AppConfig


class ReferralConfig(AppConfig):
    name = 'referral'
    verbose_name = 'Referral'

    # def ready(self):
    # from .signals import setup_signals
    # setup_signals()



from django.db import models

from .conf import settings


class Campaign(models.Model):

    campaign_id = models.CharField(max_length=255)
    status = models.CharField(
        max_length=1,
        default=settings.REFERRAL_CAMPAIGN_STATUS_DEFAULT,
        choices=settings.REFERRAL_CAMPAIGN_STATUS_CHOICES,
    )
    name = models.CharField(max_length=255)
    users = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        related_name='referrals',
        verbose_name='user',
        blank=True,
    )

    def __str__(self):
        return '{} [ {} ]'.format(self.name, self.campaign_id)

    @property
    def total_users(self):
        return self.users.count()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2019-08-28 17:54
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Campaign',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('campaign_id', models.CharField(max_length=255)),
                ('status', models.CharField(choices=[('A', 'Active'), ('I', 'Inactive')], default='A', max_length=1)),
                ('name', models.CharField(max_length=255)),
                ('users', models.ManyToManyField(
                    blank=True, null=True,
                    related_name='referrals', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
        ),
    ]



import logging

from django.contrib.auth import get_user_model
from django.http import HttpResponse

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.permissions import AllowAny

from ...models import Campaign
from ...signals_define import referral_reward_acquired

logger = logging.getLogger('referral')


class RewardAPIView(APIView):
    permission_classes = (AllowAny,)

    def post(self, request, *args, **kwargs):
        status_code = status.HTTP_404_NOT_FOUND
        webhook_data = request.data

        try:
            campaign = Campaign.objects.get(campaign_id=webhook_data.get('campaignId'))
            user = campaign.users.get(email=webhook_data.get('email'))

            referral_reward_acquired.send(
                sender=campaign,
                user=user,
                reward_data=webhook_data,
            )
            status_code = status.HTTP_200_OK
            logger.info('RewardAPIView OK')

        except Campaign.DoesNotExist:
            logger.error('Campaign.DoesNotExist')

        except get_user_model().DoesNotExist:
            logger.error('User.DoesNotExist')

        return HttpResponse(status=status_code)



import logging

from appconf import AppConf

from django.conf import settings  # noqa

logger = logging.getLogger(__name__)


class ReferralConfig(AppConf):
    APP_NAME = 'referral'

    CAMPAIGN_STATUS_ACTIVE = 'A'
    CAMPAIGN_STATUS_INACTIVE = 'I'
    CAMPAIGN_STATUS_DEFAULT = CAMPAIGN_STATUS_ACTIVE

    CAMPAIGN_STATUS_CHOICES = (
        (CAMPAIGN_STATUS_ACTIVE, 'Active'),
        (CAMPAIGN_STATUS_INACTIVE, 'Inactive'),
    )

    # Webhooks type names
    NEW_REWARD_AWARED = 'NewReward'



import factory

from faker import Faker

from referral.models import Campaign

faker = Faker()


class FakeCampaignFactory(factory.django.DjangoModelFactory):

    class Meta:
        model = Campaign

    campaign_id = factory.LazyAttribute(lambda x: faker.sha1())
    name = factory.LazyAttribute(lambda x: faker.word())



from django.conf.urls import url

from .views import CampaignViewSet, RewardAPIView


app_name = 'referral'


urlpatterns = [
    url(r'^campaigns/$', CampaignViewSet.as_view(), name='campaign-list'),
    url(r'^webhooks/2XliDq2pdp1szRJ0LR9s2598G/$', RewardAPIView.as_view(), name='reward-awared'),
]
